@@ -161,7 +161,15 @@ class Selection(Char):
 
 
 class Reference(Char):
-    pass
+
+    @staticmethod
+    def sql_format(value):
+        if not isinstance(value, basestring):
+            try:
+                value = '%s,%s' % tuple(value)
+            except TypeError:
+                pass
+        return Char.sql_format(value)
 
 
 class Many2One(Field):
