@@ -22,9 +22,9 @@ CREATE TABLE ir_ui_view (
     id INTEGER PRIMARY KEY AUTOINCREMENT,
     model VARCHAR,
     "type" VARCHAR,
-    arch TEXT,
+    data TEXT,
     field_childs VARCHAR,
-    priority INTEGER DEFAULT 0
+    priority INTEGER
 );
 
 CREATE TABLE ir_ui_menu (
@@ -40,7 +40,7 @@ CREATE TABLE ir_translation (
     src TEXT,
     src_md5 VARCHAR(32) NOT NULL,
     name VARCHAR,
-    res_id INTEGER DEFAULT 0,
+    res_id INTEGER,
     value TEXT,
     "type" VARCHAR,
     module VARCHAR,
@@ -83,13 +83,8 @@ CREATE TABLE ir_module_module (
     create_date TIMESTAMP,
     write_date TIMESTAMP,
     write_uid INTEGER,
-    website VARCHAR,
     name VARCHAR,
-    author VARCHAR,
-    url VARCHAR,
-    state VARCHAR,
-    shortdesc VARCHAR,
-    description TEXT
+    state VARCHAR
 );
 
 CREATE TABLE ir_module_module_dependency (
