@@ -13,6 +13,10 @@ from ..transaction import Transaction
 from ..pyson import Eval
 from ..pool import Pool
 
+__all__ = [
+    'Attachment',
+    ]
+
 
 def firstline(description):
     try:
@@ -23,8 +27,7 @@ def firstline(description):
 
 class Attachment(ModelSQL, ModelView):
     "Attachment"
-    _name = 'ir.attachment'
-    _description = __doc__
+    __name__ = 'ir.attachment'
     name = fields.Char('Name', required=True)
     type = fields.Selection([
         ('data', 'Data'),
@@ -35,7 +38,7 @@ class Attachment(ModelSQL, ModelView):
                 }, depends=['type']), 'get_data', setter='set_data')
     description = fields.Text('Description')
     summary = fields.Function(fields.Char('Summary',
-        on_change_with=['description']), 'get_summary')
+        on_change_with=['description']), 'on_change_with_summary')
     resource = fields.Reference('Resource', selection='models_get',
         select=True)
     link = fields.Char('Link', states={
@@ -47,23 +50,25 @@ class Attachment(ModelSQL, ModelView):
                 'invisible': Eval('type') != 'data',
                 }, depends=['type']), 'get_data')
     last_modification = fields.Function(fields.DateTime('Last Modification'),
-            'get_last_modification')
+        'get_last_modification')
     last_user = fields.Function(fields.Char('Last User'),
         'get_last_user')
 
-    def __init__(self):
-        super(Attachment, self).__init__()
-        self._sql_constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(Attachment, cls).__setup__()
+        cls._sql_constraints += [
             ('resource_name', 'UNIQUE(resource, name)',
                 'The  names of attachments must be unique by resource!'),
         ]
 
-    def init(self, module_name):
+    @classmethod
+    def __register__(cls, module_name):
         cursor = Transaction().cursor
 
-        super(Attachment, self).init(module_name)
+        super(Attachment, cls).__register__(module_name)
 
-        table = TableHandler(cursor, self, module_name)
+        table = TableHandler(cursor, cls, module_name)
 
         # Migration from 1.4 res_model and res_id merged into resource
         # Reference
@@ -71,58 +76,61 @@ class Attachment(ModelSQL, ModelView):
                 table.column_exist('res_id'):
             table.drop_constraint('res_model_res_id_name')
             cursor.execute('UPDATE "%s" '
-            'SET "resource" = "res_model"||\',\'||"res_id"' % self._table)
+            'SET "resource" = "res_model"||\',\'||"res_id"' % cls._table)
             table.drop_column('res_model')
             table.drop_column('res_id')
 
-    def default_type(self):
+    @staticmethod
+    def default_type():
         return 'data'
 
-    def default_resource(self):
+    @staticmethod
+    def default_resource():
         return Transaction().context.get('resource')
 
-    def default_collision(self):
+    @staticmethod
+    def default_collision():
         return 0
 
-    def models_get(self):
+    @staticmethod
+    def models_get():
         pool = Pool()
-        model_obj = pool.get('ir.model')
-        model_ids = model_obj.search([])
+        Model = pool.get('ir.model')
+        models = Model.search([])
         res = []
-        for model in model_obj.browse(model_ids):
+        for model in models:
             res.append([model.model, model.name])
         return res
 
-    def get_data(self, ids, name):
-        res = {}
+    def get_data(self, name):
         db_name = Transaction().cursor.dbname
-        format_ = Transaction().context.pop('%s.%s' % (self._name, name), '')
-        for attachment in self.browse(ids):
-            value = None
+        format_ = Transaction().context.pop('%s.%s'
+            % (self.__name__, name), '')
+        value = None
+        if name == 'data_size' or format_ == 'size':
+            value = 0
+        if self.digest:
+            filename = self.digest
+            if self.collision:
+                filename = filename + '-' + str(self.collision)
+            filename = os.path.join(CONFIG['data_path'], db_name,
+                    filename[0:2], filename[2:4], filename)
             if name == 'data_size' or format_ == 'size':
-                value = 0
-            if attachment.digest:
-                filename = attachment.digest
-                if attachment.collision:
-                    filename = filename + '-' + str(attachment.collision)
-                filename = os.path.join(CONFIG['data_path'], db_name,
-                        filename[0:2], filename[2:4], filename)
-                if name == 'data_size' or format_ == 'size':
-                    try:
-                        statinfo = os.stat(filename)
-                        value = statinfo.st_size
-                    except OSError:
-                        pass
-                else:
-                    try:
-                        with open(filename, 'rb') as file_p:
-                            value = buffer(file_p.read())
-                    except IOError:
-                        pass
-            res[attachment.id] = value
-        return res
+                try:
+                    statinfo = os.stat(filename)
+                    value = statinfo.st_size
+                except OSError:
+                    pass
+            else:
+                try:
+                    with open(filename, 'rb') as file_p:
+                        value = buffer(file_p.read())
+                except IOError:
+                    pass
+        return value
 
-    def set_data(self, ids, name, value):
+    @classmethod
+    def set_data(cls, attachments, name, value):
         if value is None:
             return
         cursor = Transaction().cursor
@@ -168,77 +176,73 @@ class Attachment(ModelSQL, ModelView):
         else:
             with open(filename, 'wb') as file_p:
                 file_p.write(value)
-        self.write(ids, {
+        cls.write(attachments, {
             'digest': digest,
             'collision': collision,
             })
 
-    def get_summary(self, ids, name):
-        return dict((x.id, firstline(x.description or ''))
-            for x in self.browse(ids))
+    def on_change_with_summary(self, name=None):
+        return firstline(self.description or '')
 
-    def on_change_with_summary(self, values):
-        return firstline(values.get('description') or '')
+    def get_last_modification(self, name):
+        return (self.write_date if self.write_date else self.create_date
+            ).replace(microsecond=0)
 
-    def get_last_modification(self, ids, name):
-        return dict((x.id, (x.write_date if x.write_date else x.create_date
-            ).replace(microsecond=0))
-            for x in self.browse(ids))
-
-    def get_last_user(self, ids, name):
+    @classmethod
+    def get_last_user(cls, attachments, name):
         with Transaction().set_user(0):
-            return dict((x.id, x.write_uid.rec_name if x.write_uid
-                else x.create_uid.rec_name) for x in self.browse(ids))
+            return dict(
+                (x.id, x.write_uid.rec_name
+                    if x.write_uid else x.create_uid.rec_name)
+                for x in cls.browse(attachments))
 
-    def check_access(self, ids, mode='read'):
+    @classmethod
+    def check_access(cls, ids, mode='read'):
         pool = Pool()
-        model_access_obj = pool.get('ir.model.access')
+        ModelAccess = pool.get('ir.model.access')
         if Transaction().user == 0:
             return
-        if not ids:
-            return
-        if isinstance(ids, (int, long)):
-            ids = [ids]
         model_names = set()
         with Transaction().set_user(0):
-            for attachment in self.browse(ids):
+            for attachment in cls.browse(ids):
                 if attachment.resource:
-                    model_names.add(attachment.resource.split(',')[0])
+                    model_names.add(attachment.resource.__name__)
         for model_name in model_names:
-            model_access_obj.check(model_name, mode=mode)
+            ModelAccess.check(model_name, mode=mode)
 
-    def read(self, ids, fields_names=None):
-        self.check_access(ids, mode='read')
-        return super(Attachment, self).read(ids, fields_names=fields_names)
+    @classmethod
+    def read(cls, ids, fields_names=None):
+        cls.check_access(ids, mode='read')
+        return super(Attachment, cls).read(ids, fields_names=fields_names)
 
-    def delete(self, ids):
-        self.check_access(ids, mode='delete')
-        return super(Attachment, self).delete(ids)
+    @classmethod
+    def delete(cls, attachments):
+        cls.check_access([a.id for a in attachments], mode='delete')
+        super(Attachment, cls).delete(attachments)
 
-    def write(self, ids, vals):
-        self.check_access(ids, mode='write')
-        res = super(Attachment, self).write(ids, vals)
-        self.check_access(ids, mode='write')
-        return res
+    @classmethod
+    def write(cls, attachments, vals):
+        cls.check_access([a.id for a in attachments], mode='write')
+        super(Attachment, cls).write(attachments, vals)
+        cls.check_access(attachments, mode='write')
 
-    def create(self, vals):
-        res = super(Attachment, self).create(vals)
-        self.check_access(res, mode='create')
-        return res
+    @classmethod
+    def create(cls, vals):
+        attachment = super(Attachment, cls).create(vals)
+        cls.check_access([attachment.id], mode='create')
+        return attachment
 
-    def view_header_get(self, value, view_type='form'):
+    @classmethod
+    def view_header_get(cls, value, view_type='form'):
         pool = Pool()
-        ir_model_obj = pool.get('ir.model')
-        value = super(Attachment, self).view_header_get(value,
+        Model = pool.get('ir.model')
+        value = super(Attachment, cls).view_header_get(value,
                 view_type=view_type)
         resource = Transaction().context.get('resource')
         if resource:
             model_name, record_id = resource.split(',', 1)
-            ir_model_id, = ir_model_obj.search([('model', '=', model_name)])
-            ir_model = ir_model_obj.browse(ir_model_id)
-            model_obj = pool.get(model_name)
-            record = model_obj.browse(int(record_id))
+            ir_model, = Model.search([('model', '=', model_name)])
+            Resource = pool.get(model_name)
+            record = Resource(int(record_id))
             value = '%s - %s - %s' % (ir_model.name, record.rec_name, value)
         return value
-
-Attachment()
