@@ -11,11 +11,14 @@ from ..transaction import Transaction
 from ..cache import Cache
 from ..pool import Pool
 
+__all__ = [
+    'Trigger', 'TriggerLog',
+    ]
+
 
 class Trigger(ModelSQL, ModelView):
     "Trigger"
-    _name = 'ir.trigger'
-    _description = __doc__
+    __name__ = 'ir.trigger'
     name = fields.Char('Name', required=True, translate=True)
     active = fields.Boolean('Active', select=True)
     model = fields.Many2One('ir.model', 'Model', required=True, select=True)
@@ -48,41 +51,46 @@ class Trigger(ModelSQL, ModelView):
         '0 for no delay.')
     action_model = fields.Many2One('ir.model', 'Action Model', required=True)
     action_function = fields.Char('Action Function', required=True)
+    _get_triggers_cache = Cache('ir_trigger.get_triggers')
 
-    def __init__(self):
-        super(Trigger, self).__init__()
-        self._sql_constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(Trigger, cls).__setup__()
+        cls._sql_constraints += [
             ('on_exclusive',
                 'CHECK(NOT(on_time AND (on_create OR on_write OR on_delete)))',
                 '"On Time" and others are mutually exclusive!'),
         ]
-        self._constraints += [
+        cls._constraints += [
             ('check_condition', 'invalid_condition'),
         ]
-        self._error_messages.update({
+        cls._error_messages.update({
             'invalid_condition': 'Condition must be a python expression!',
         })
-        self._order.insert(0, ('name', 'ASC'))
+        cls._order.insert(0, ('name', 'ASC'))
 
-    def check_condition(self, ids):
+    @classmethod
+    def check_condition(cls, triggers):
         '''
         Check condition
         '''
-        for trigger in self.browse(ids):
+        for trigger in triggers:
             try:
                 compile(trigger.condition, '', 'eval')
             except (SyntaxError, TypeError):
                 return False
         return True
 
-    def default_active(self):
+    @staticmethod
+    def default_active():
         return True
 
-    def default_limit_number(self):
+    @staticmethod
+    def default_limit_number():
         return 0
 
-    def on_change_on_time(self, values):
-        if values.get('on_time'):
+    def on_change_on_time(self):
+        if self.on_time:
             return {
                     'on_create': False,
                     'on_write': False,
@@ -90,35 +98,31 @@ class Trigger(ModelSQL, ModelView):
                     }
         return {}
 
-    def on_change_on_create(self, values):
-        if values.get('on_create'):
+    def on_change_on_create(self):
+        if self.on_create:
             return {
                     'on_time': False,
                     }
         return {}
 
-    def on_change_on_write(self, values):
-        if values.get('on_write'):
+    def on_change_on_write(self):
+        if self.on_write:
             return {
                     'on_time': False,
                     }
         return {}
 
-    def on_change_on_delete(self, values):
-        if values.get('on_delete'):
+    def on_change_on_delete(self):
+        if self.on_delete:
             return {
                     'on_time': False,
                     }
         return {}
 
-    @Cache('ir_trigger.get_triggers')
-    def get_triggers(self, model_name, mode):
+    @classmethod
+    def get_triggers(cls, model_name, mode):
         """
-        Return trigger ids for a model and a mode
-
-        :param model_name: the name of the model
-        :param mode: the mode that can be 'create', 'write', 'delete' or 'time'
-        :return: a list of ir.trigger ids
+        Return triggers for a model and a mode
         """
         assert mode in ['create', 'write', 'delete', 'time'], \
                 'Invalid trigger mode'
@@ -126,19 +130,22 @@ class Trigger(ModelSQL, ModelView):
         if Transaction().user == 0:
             return []  # XXX is it want we want?
 
-        trigger_ids = self.search([
-            ('model.model', '=', model_name),
-            ('on_%s' % mode, '=', True),
-            ])
-        return trigger_ids
+        key = (model_name, mode)
+        trigger_ids = cls._get_triggers_cache.get(key)
+        if trigger_ids is not None:
+            return cls.browse(trigger_ids)
 
-    def eval(self, trigger, record):
+        triggers = cls.search([
+                ('model.model', '=', model_name),
+                ('on_%s' % mode, '=', True),
+                ])
+        cls._get_triggers_cache.set(key, map(int, triggers))
+        return triggers
+
+    @staticmethod
+    def eval(trigger, record):
         """
         Evaluate the condition of trigger
-
-        :param trigger: a BrowseRecord of ir.trigger
-        :param record: a BrowseRecord of the tested model
-        :return: a boolean
         """
         env = {}
         env['current_date'] = datetime.datetime.today()
@@ -147,18 +154,16 @@ class Trigger(ModelSQL, ModelView):
         env['self'] = record
         return bool(safe_eval(trigger.condition, env))
 
-    def trigger_action(self, ids, trigger_id):
+    @classmethod
+    def trigger_action(cls, records, trigger):
         """
-        Trigger the action define on trigger_id for the ids
-
-        :param ids: the list of record ids triggered
-        :param trigger_id: the trigger id
+        Trigger the action define on trigger for the records
         """
         pool = Pool()
-        trigger_log_obj = pool.get('ir.trigger.log')
-        trigger = self.browse(trigger_id)
-        model_obj = pool.get(trigger.action_model.model)
+        TriggerLog = pool.get('ir.trigger.log')
+        Model = pool.get(trigger.action_model.model)
         cursor = Transaction().cursor
+        ids = map(int, records)
 
         # Filter on limit_number
         if trigger.limit_number:
@@ -169,7 +174,7 @@ class Trigger(ModelSQL, ModelView):
                 cursor.execute('SELECT "record_id", COUNT(1) FROM "%s" '
                         'WHERE %s AND "trigger" = %%s '
                         'GROUP BY "record_id"'
-                        % (trigger_log_obj._table, red_sql),
+                        % (TriggerLog._table, red_sql),
                         red_ids + [trigger.id])
                 number = dict(cursor.fetchall())
                 for record_id in sub_ids:
@@ -190,7 +195,7 @@ class Trigger(ModelSQL, ModelView):
                         'FROM "%s" '
                         'WHERE %s AND "trigger" = %%s '
                         'GROUP BY "record_id"'
-                        % (trigger_log_obj._table, red_sql),
+                        % (TriggerLog._table, red_sql),
                         red_ids + [trigger.id])
                 delay = dict(cursor.fetchall())
                 for record_id in sub_ids:
@@ -214,66 +219,66 @@ class Trigger(ModelSQL, ModelView):
                         new_ids.append(record_id)
             ids = new_ids
 
-        if ids:
-            getattr(model_obj, trigger.action_function)(ids, trigger_id)
+        records = Model.browse(ids)
+        if records:
+            getattr(Model, trigger.action_function)(records, trigger)
         if trigger.limit_number or trigger.minimum_delay:
-            for record_id in ids:
-                trigger_log_obj.create({
+            for record in records:
+                TriggerLog.create({
                     'trigger': trigger.id,
-                    'record_id': record_id,
+                    'record_id': record.id,
                     })
 
-    def trigger_time(self):
+    @classmethod
+    def trigger_time(cls):
         '''
         Trigger time actions
         '''
         pool = Pool()
-        trigger_ids = self.search([
-            ('on_time', '=', True),
-            ])
-        for trigger in self.browse(trigger_ids):
-            model_obj = pool.get(trigger.model.model)
-            triggered_ids = []
+        triggers = cls.search([
+                ('on_time', '=', True),
+                ])
+        for trigger in triggers:
+            Model = pool.get(trigger.model.model)
+            triggered = []
             # TODO add a domain
-            record_ids = model_obj.search([])
-            for record in model_obj.browse(record_ids):
-                if self.eval(trigger, record):
-                    triggered_ids.append(record.id)
-            if triggered_ids:
-                self.trigger_action(triggered_ids, trigger.id)
+            records = Model.search([])
+            for record in records:
+                if cls.eval(trigger, record):
+                    triggered.append(record)
+            if triggered:
+                cls.trigger_action(triggered, trigger)
 
-    def create(self, values):
-        res = super(Trigger, self).create(values)
+    @classmethod
+    def create(cls, values):
+        res = super(Trigger, cls).create(values)
         # Restart the cache on the get_triggers method of ir.trigger
-        self.get_triggers.reset()
+        cls._get_triggers_cache.clear()
         return res
 
-    def write(self, ids, values):
-        res = super(Trigger, self).write(ids, values)
+    @classmethod
+    def write(cls, ids, values):
+        res = super(Trigger, cls).write(ids, values)
         # Restart the cache on the get_triggers method of ir.trigger
-        self.get_triggers.reset()
+        cls._get_triggers_cache.clear()
         return res
 
-    def delete(self, ids):
-        res = super(Trigger, self).delete(ids)
+    @classmethod
+    def delete(cls, records):
+        super(Trigger, cls).delete(records)
         # Restart the cache on the get_triggers method of ir.trigger
-        self.get_triggers.reset()
-        return res
-
-Trigger()
+        cls._get_triggers_cache.clear()
 
 
 class TriggerLog(ModelSQL):
     'Trigger Log'
-    _name = 'ir.trigger.log'
-    _description = __doc__
+    __name__ = 'ir.trigger.log'
     trigger = fields.Many2One('ir.trigger', 'Trigger', required=True)
     record_id = fields.Integer('Record ID', required=True)
 
-    def init(self, module_name):
-        super(TriggerLog, self).init(module_name)
+    @classmethod
+    def __register__(cls, module_name):
+        super(TriggerLog, cls).__register__(module_name)
 
-        table = TableHandler(Transaction().cursor, self, module_name)
+        table = TableHandler(Transaction().cursor, cls, module_name)
         table.index_action(['trigger', 'record_id'], 'add')
-
-TriggerLog()
