@@ -57,6 +57,14 @@ def context_validate(value):
     assert isinstance(value, dict), 'context must be a dict'
 
 
+def size_validate(value):
+    if value is not None:
+        assert isinstance(value, (int, PYSON)), 'size must be PYSON'
+        if hasattr(value, 'types'):
+            assert value.types() == set([int]), \
+                'size must return integer'
+
+
 class Field(object):
     _type = None
 
@@ -113,6 +121,7 @@ class Field(object):
         assert loading in ('lazy', 'eager'), \
             'loading must be "lazy" or "eager"'
         self.loading = loading
+        self.name = None
 
     def _get_domain(self):
         return self.__domain
@@ -166,3 +175,15 @@ class Field(object):
         self.__context = value
 
     context = property(_get_context, _set_context)
+
+    def __get__(self, inst, cls):
+        if inst is None:
+            return self
+        assert self.name is not None
+        return inst.__getattr__(self.name)
+
+    def __set__(self, inst, value):
+        assert self.name is not None
+        if inst._values is None:
+            inst._values = {}
+        inst._values[self.name] = value
