@@ -8,6 +8,10 @@ import contextlib
 from functools import reduce
 import imp
 import operator
+import ConfigParser
+from glob import iglob
+import datetime
+
 import trytond.tools as tools
 from trytond.config import CONFIG
 from trytond.transaction import Transaction
@@ -107,9 +111,6 @@ class Node(Singleton):
 
     def __setattr__(self, name, value):
         super(Node, self).__setattr__(name, value)
-        if name in ('init', 'update'):
-            for child in self.childs:
-                setattr(child, name, value)
         if name == 'depth':
             for child in self.childs:
                 setattr(child, name, value + 1)
@@ -128,35 +129,32 @@ class Node(Singleton):
         return res
 
 
+def get_module_info(name):
+    "Return the content of the tryton.cfg"
+    config = ConfigParser.ConfigParser()
+    with tools.file_open(os.path.join(name, 'tryton.cfg')) as fp:
+        config.readfp(fp)
+        directory = os.path.dirname(fp.name)
+    info = dict(config.items('tryton'))
+    info['directory'] = directory
+    for key in ('depends', 'extras_depend', 'xml'):
+        if key in info:
+            info[key] = info[key].strip().splitlines()
+    return info
+
+
 def create_graph(module_list):
     graph = Graph()
     packages = []
 
     for module in module_list:
-        tryton_file = OPJ(MODULES_PATH, module, '__tryton__.py')
-        mod_path = OPJ(MODULES_PATH, module)
-        if module in ('ir', 'res', 'webdav', 'test'):
-            root_path = os.path.abspath(os.path.dirname(
-                    os.path.dirname(__file__)))
-            tryton_file = OPJ(root_path, module, '__tryton__.py')
-            mod_path = OPJ(root_path, module)
-        elif module in EGG_MODULES:
-            ep = EGG_MODULES[module]
-            tryton_file = OPJ(ep.dist.location, 'trytond', 'modules', module,
-                    '__tryton__.py')
-            mod_path = OPJ(ep.dist.location, 'trytond', 'modules', module)
-            if not os.path.isfile(tryton_file) or not os.path.isdir(mod_path):
-                # When testing modules from setuptools location is the module
-                # directory
-                tryton_file = OPJ(ep.dist.location, '__tryton__.py')
-                mod_path = os.path.dirname(ep.dist.location)
-        if os.path.isfile(tryton_file):
-            with tools.file_open(tryton_file, subdir='') as fp:
-                info = tools.safe_eval(fp.read())
-            packages.append((module, info.get('depends', []),
-                    info.get('extras_depend', []), info))
-        elif module != 'all':
-            raise Exception('Module %s not found' % module)
+        try:
+            info = get_module_info(module)
+        except IOError:
+            if module != 'all':
+                raise Exception('Module %s not found' % module)
+        packages.append((module, info.get('depends', []),
+                info.get('extras_depend', []), info))
 
     current, later = set([x[0] for x in packages]), set()
     all_packages = set(current)
@@ -217,17 +215,21 @@ def load_module_graph(graph, pool, lang=None):
             continue
         logger.info(module)
         sys.stdout.flush()
-        objects = pool.instanciate(module)
+        classes = pool.setup(module)
         package_state = module2state.get(module, 'uninstalled')
         if (is_module_to_install(module)
                 or package_state in ('to install', 'to upgrade')):
-            for type in objects.keys():
-                for obj in objects[type]:
-                    logger.info('%s:init %s' % (module, obj._name))
-                    obj.init(module)
-            for model in objects['model']:
+            if package_state not in ('to install', 'to upgrade'):
+                package_state = 'to install'
+            for child in package.childs:
+                module2state[child.name] = package_state
+            for type in classes.keys():
+                for cls in classes[type]:
+                    logger.info('%s:register %s' % (module, cls.__name__))
+                    cls.__register__(module)
+            for model in classes['model']:
                 if hasattr(model, '_history'):
-                    models_to_update_history.add(model._name)
+                    models_to_update_history.add(model.__name__)
 
             #Instanciate a new parser for the package:
             tryton_parser = convert.TrytondXmlHandler(pool=pool, module=module)
@@ -241,42 +243,51 @@ def load_module_graph(graph, pool, lang=None):
 
             modules_todo.append((module, list(tryton_parser.to_delete)))
 
-            for filename in package.info.get('translation', []):
+            for filename in iglob('%s/%s/*.po'
+                    % (package.info['directory'], 'locale')):
                 filename = filename.replace('/', os.sep)
                 lang2 = os.path.splitext(os.path.basename(filename))[0]
                 if lang2 not in lang:
                     continue
-                logger.info('%s:loading %s' % (module, filename))
-                with tools.file_open(OPJ(module, filename)) as trans_file:
-                    po_path = trans_file.name
-                translation_obj = pool.get('ir.translation')
-                translation_obj.translation_import(lang2, module, po_path)
+                logger.info('%s:loading %s' % (module,
+                        filename[len(package.info['directory']) + 1:]))
+                Translation = pool.get('ir.translation')
+                Translation.translation_import(lang2, module, filename)
 
-            cursor.execute("UPDATE ir_module_module SET state = 'installed' " \
-                    "WHERE name = %s", (package.name,))
+            cursor.execute('SELECT id FROM ir_module_module '
+                'WHERE name = %s', (package.name,))
+            try:
+                module_id, = cursor.fetchone()
+                cursor.execute('UPDATE ir_module_module SET state = %s '
+                    'WHERE id = %s', ('installed', module_id))
+            except TypeError:
+                cursor.execute('INSERT INTO ir_module_module '
+                    '(create_uid, create_date, name, state) '
+                    'VALUES (%s, %s, %s, %s)', (0, datetime.datetime.now(),
+                        package.name, 'installed'))
             module2state[package.name] = 'installed'
 
         cursor.commit()
 
     # Create missing reports
     from trytond.report import Report
-    report_obj = pool.get('ir.action.report')
-    report_ids = report_obj.search([
-        ('module', '=', module),
-        ])
+    ActionReport = pool.get('ir.action.report')
+    reports = ActionReport.search([
+            ('module', '=', module),
+            ])
     report_names = pool.object_name_list(type='report')
-    for report in report_obj.browse(report_ids):
+    for report in reports:
         report_name = report.report_name
         if report_name not in report_names:
             report = object.__new__(Report)
-            report._name = report_name
+            report.__name__ = report_name
             pool.add(report, type='report')
             report.__init__()
 
     for model_name in models_to_update_history:
         model = pool.get(model_name)
         if model._history:
-            logger.info('history:update %s' % model._name)
+            logger.info('history:update %s' % model.__name__)
             model._update_history_table()
 
     # Vacuum :
@@ -309,9 +320,13 @@ def register_classes():
     Import modules to register the classes in the Pool
     '''
     import trytond.ir
+    trytond.ir.register()
     import trytond.res
+    trytond.res.register()
     import trytond.webdav
+    trytond.webdav.register()
     import trytond.test
+    trytond.test.register()
     logger = logging.getLogger('modules')
 
     for package in create_graph(get_module_list())[0]:
@@ -325,12 +340,13 @@ def register_classes():
         if os.path.isdir(OPJ(MODULES_PATH, module)):
             mod_file, pathname, description = imp.find_module(module,
                     [MODULES_PATH])
-            try:
-                imp.load_module('trytond.modules.' + module, mod_file,
-                        pathname, description)
-            finally:
-                if mod_file is not None:
-                    mod_file.close()
+            the_module = imp.load_module(
+                'trytond.modules.' + module, mod_file, pathname, description)
+            # Some modules register nothing in the Pool
+            if hasattr(the_module, 'register'):
+                the_module.register()
+            if mod_file is not None:
+                mod_file.close()
         elif module in EGG_MODULES:
             ep = EGG_MODULES[module]
             mod_path = os.path.join(ep.dist.location,
@@ -350,7 +366,7 @@ def register_classes():
                     [mod_path])
             try:
                 imp.load_module('trytond.modules.' + module, mod_file,
-                        pathname, description)
+                        pathname, description).register()
             finally:
                 if mod_file is not None:
                     mod_file.close()
@@ -410,15 +426,15 @@ def load_modules(database_name, pool, update=False, lang=None):
                             'WHERE module = %s ' \
                             'ORDER BY id DESC', (mod_name,))
                     for rmod, rid in cursor.fetchall():
-                        pool.get(rmod).delete(rid)
+                        pool.get(rmod).delete([rmod(rid)])
                     cursor.commit()
                 cursor.execute("UPDATE ir_module_module SET state = %s " \
                         "WHERE state IN ('to remove')", ('uninstalled',))
                 cursor.commit()
                 res = False
 
-            module_obj = pool.get('ir.module.module')
-            module_obj.update_list()
+            Module = pool.get('ir.module.module')
+            Module.update_list()
         cursor.commit()
     Cache.resets(database_name)
     return res
