@@ -306,6 +306,18 @@ class SimpleJSONRPCServer(SocketServer.TCPServer,
         for handler in self.handlers:
             self.shutdown_request(handler.request)
 
+    if sys.version_info[:2] <= (2, 6):
+
+        def shutdown_request(self, request):
+            """Called to shutdown and close an individual request."""
+            try:
+                #explicitly shutdown.  socket.close() merely releases
+                #the socket and waits for GC to perform the actual close.
+                request.shutdown(socket.SHUT_WR)
+            except socket.error:
+                pass  # some platforms may raise ENOTCONN here
+            self.close_request(request)
+
 
 class SimpleThreadedJSONRPCServer(SocketServer.ThreadingMixIn,
         SimpleJSONRPCServer):
@@ -317,6 +329,8 @@ class SimpleThreadedJSONRPCServer(SocketServer.ThreadingMixIn,
                 socket.SO_REUSEADDR, 1)
         self.socket.setsockopt(socket.SOL_SOCKET,
             socket.SO_KEEPALIVE, 1)
+        self.socket.setsockopt(socket.IPPROTO_TCP,
+            socket.TCP_NODELAY, 1)
         SimpleJSONRPCServer.server_bind(self)
 
 
