@@ -10,15 +10,20 @@ except ImportError:
     hashlib = None
     import sha
 import time
+import datetime
+from itertools import groupby, ifilter
+from operator import attrgetter
+
 from ..model import ModelView, ModelSQL, fields
 from ..wizard import Wizard, StateView, Button, StateTransition
-from ..tools import safe_eval, reduce_ids
+from ..tools import safe_eval
 from ..backend import TableHandler
 from ..transaction import Transaction
 from ..cache import Cache
 from ..pool import Pool
 from ..config import CONFIG
 from ..pyson import PYSONEncoder
+from ..rpc import RPC
 
 try:
     import pytz
@@ -26,11 +31,15 @@ try:
 except ImportError:
     TIMEZONES = []
 
+__all__ = [
+    'User', 'UserAction', 'UserGroup', 'Warning_',
+    'UserConfigStart', 'UserConfig',
+    ]
+
 
 class User(ModelSQL, ModelView):
     "User"
-    _name = "res.user"
-    _description = __doc__
+    __name__ = "res.user"
     name = fields.Char('Name', required=True, select=True, translate=True)
     login = fields.Char('Login', required=True)
     login_try = fields.Integer('Login Try', required=True)
@@ -59,21 +68,25 @@ class User(ModelSQL, ModelView):
     email = fields.Char('Email')
     status_bar = fields.Function(fields.Char('Status Bar'), 'get_status_bar')
     warnings = fields.One2Many('res.user.warning', 'user', 'Warnings')
-    connections = fields.Function(fields.Integer('Connections'),
-            'get_connections')
+    sessions = fields.Function(fields.Integer('Sessions'),
+            'get_sessions')
+    _get_preferences_cache = Cache('res_user.get_preferences')
+    _get_groups_cache = Cache('res_user.get_groups')
+    _get_login_cache = Cache('res_user._get_login', context=False)
 
-    def __init__(self):
-        super(User, self).__init__()
-        self._rpc.update({
-            'get_preferences': False,
-            'set_preferences': True,
-            'get_preferences_fields_view': False,
-        })
-        self._sql_constraints += [
+    @classmethod
+    def __setup__(cls):
+        super(User, cls).__setup__()
+        cls.__rpc__.update({
+                'get_preferences': RPC(),
+                'set_preferences': RPC(readonly=False),
+                'get_preferences_fields_view': RPC(),
+                })
+        cls._sql_constraints += [
             ('login_key', 'UNIQUE (login)',
                 'You can not have two users with the same login!')
         ]
-        self._preferences_fields = [
+        cls._preferences_fields = [
             'name',
             'password',
             'email',
@@ -84,13 +97,13 @@ class User(ModelSQL, ModelView):
             'status_bar',
             'warnings',
         ]
-        self._context_fields = [
+        cls._context_fields = [
             'language',
             'language_direction',
             'timezone',
             'groups',
         ]
-        self._error_messages.update({
+        cls._error_messages.update({
             'rm_root': 'You can not remove the root user\n' \
                             'as it is used internally for resources\n' \
                             'created by the system ' \
@@ -98,9 +111,10 @@ class User(ModelSQL, ModelView):
             'wrong_password': 'Wrong password!',
             })
 
-    def init(self, module_name):
-        super(User, self).init(module_name)
-        table = TableHandler(Transaction().cursor, self, module_name)
+    @classmethod
+    def __register__(cls, module_name):
+        super(User, cls).__register__(module_name)
+        table = TableHandler(Transaction().cursor, cls, module_name)
 
         # Migration from 1.6
 
@@ -120,74 +134,77 @@ class User(ModelSQL, ModelView):
         # Migration from 2.2
         table.not_null_action('menu', action='remove')
 
-    def default_login_try(self):
+    @staticmethod
+    def default_login_try():
         return 0
 
-    def default_password(self):
+    @staticmethod
+    def default_password():
         return None
 
-    def default_active(self):
+    @staticmethod
+    def default_active():
         return True
 
-    def default_menu(self):
+    @staticmethod
+    def default_menu():
         pool = Pool()
-        action_obj = pool.get('ir.action')
-        action_ids = action_obj.search([
+        Action = pool.get('ir.action')
+        actions = Action.search([
             ('usage', '=', 'menu'),
             ], limit=1)
-        if action_ids:
-            return action_ids[0]
+        if actions:
+            return actions[0].id
         return None
 
-    def get_pyson_menu(self, ids, name):
+    def get_pyson_menu(self, name):
         pool = Pool()
-        action_obj = pool.get('ir.action')
-
+        Action = pool.get('ir.action')
         encoder = PYSONEncoder()
-        result = {}
-        for user in self.browse(ids):
-            result[user.id] = encoder.encode(
-                action_obj.get_action_values(user.menu.type, user.menu.id))
+        return encoder.encode(
+            Action.get_action_values(self.menu.type, [self.menu.id])[0])
+
+    def get_language_direction(self, name):
+        pool = Pool()
+        Lang = pool.get('ir.lang')
+        if self.language:
+            return self.language.direction
+        else:
+            return Lang.default_direction()
+
+    def get_status_bar(self, name):
+        return self.name
+
+    @staticmethod
+    def get_sessions(users, name):
+        Session = Pool().get('ir.session')
+        cursor = Transaction().cursor
+        now = datetime.datetime.now()
+        timeout = datetime.timedelta(seconds=int(CONFIG['session_timeout']))
+        result = dict((u.id, 0) for u in users)
+        for i in range(0, len(users), cursor.IN_MAX):
+            sub_ids = [u.id for u in users[i:i + cursor.IN_MAX]]
+
+            with Transaction().set_user(0):
+                sessions = Session.search([
+                        ('create_uid', 'in', sub_ids),
+                        ], order=[('create_uid', 'ASC')])
+
+            def filter_(session):
+                timestamp = session.write_date or session.create_date
+                return abs(timestamp - now) < timeout
+            result.update(dict((i, len(list(g)))
+                    for i, g in groupby(ifilter(filter_, sessions),
+                        attrgetter('create_uid.id'))))
         return result
 
-    def get_language_direction(self, ids, name):
-        res = {}
-        pool = Pool()
-        lang_obj = pool.get('ir.lang')
-        default_direction = lang_obj.default_direction()
-        for user in self.browse(ids):
-            if user.language:
-                res[user.id] = user.language.direction
-            else:
-                res[user.id] = default_direction
-        return res
-
-    def get_status_bar(self, ids, name):
-        res = {}
-        for user in self.browse(ids):
-            res[user.id] = user.name
-        return res
-
-    def get_connections(self, ids, name):
-        session_obj = Pool().get('ir.session')
-        cursor = Transaction().cursor
-        connections = dict((i, 0) for i in ids)
-        for i in range(0, len(ids), cursor.IN_MAX):
-            sub_ids = ids[i:i + cursor.IN_MAX]
-            red_sql, red_ids = reduce_ids('create_uid', sub_ids)
-            cursor.execute('SELECT create_uid, COUNT(1) '
-                'FROM "' + session_obj._table + '" '
-                'WHERE ' + red_sql + ' '
-                'GROUP BY create_uid', red_ids)
-            connections.update(dict(cursor.fetchall()))
-        return connections
-
-    def _convert_vals(self, vals):
+    @staticmethod
+    def _convert_vals(vals):
         vals = vals.copy()
         pool = Pool()
-        action_obj = pool.get('ir.action')
+        Action = pool.get('ir.action')
         if 'menu' in vals:
-            vals['menu'] = action_obj.get_action_id(vals['menu'])
+            vals['menu'] = Action.get_action_id(vals['menu'])
         if 'password' in vals:
             if vals['password'] == 'x' * 10:
                 del vals['password']
@@ -197,122 +214,111 @@ class User(ModelSQL, ModelView):
                 vals['password'] += vals['salt']
         return vals
 
-    def create(self, vals):
-        vals = self._convert_vals(vals)
-        res = super(User, self).create(vals)
+    @classmethod
+    def create(cls, vals):
+        vals = cls._convert_vals(vals)
+        res = super(User, cls).create(vals)
         # Restart the cache for _get_login
-        self._get_login.reset()
+        cls._get_login_cache.clear()
         return res
 
-    def write(self, ids, vals):
-        vals = self._convert_vals(vals)
-        res = super(User, self).write(ids, vals)
+    @classmethod
+    def write(cls, users, vals):
+        vals = cls._convert_vals(vals)
+        super(User, cls).write(users, vals)
         # Restart the cache for domain_get method
         pool = Pool()
-        pool.get('ir.rule').domain_get.reset()
+        pool.get('ir.rule')._domain_get_cache.clear()
         # Restart the cache for get_groups
-        self.get_groups.reset()
+        cls._get_groups_cache.clear()
         # Restart the cache for _get_login
-        self._get_login.reset()
+        cls._get_login_cache.clear()
         # Restart the cache for get_preferences
-        self.get_preferences.reset()
+        cls._get_preferences_cache.clear()
         # Restart the cache of check
-        pool.get('ir.model.access').check.reset()
+        pool.get('ir.model.access')._get_access_cache.clear()
         # Restart the cache
-        for _, model in pool.iterobject():
-            try:
-                model.fields_view_get.reset()
-            except Exception:
-                pass
-        return res
+        ModelView._fields_view_get_cache.clear()
 
-    def delete(self, ids):
-        if isinstance(ids, (int, long)):
-            ids = [ids]
-        if 0 in ids:
-            self.raise_user_error('rm_root')
-        res = super(User, self).delete(ids)
+    @classmethod
+    def delete(cls, users):
+        if [u.id for u in users if u.id == 0]:
+            cls.raise_user_error('rm_root')
+        super(User, cls).delete(users)
         # Restart the cache for _get_login
-        self._get_login.reset()
+        cls._get_login_cache.clear()
+
+    @classmethod
+    def read(cls, ids, fields_names=None):
+        res = super(User, cls).read(ids, fields_names=fields_names)
+        for val in res:
+            if 'password' in val:
+                val['password'] = 'x' * 10
         return res
 
-    def read(self, ids, fields_names=None):
-        res = super(User, self).read(ids, fields_names=fields_names)
-        if isinstance(ids, (int, long)):
-            if 'password' in res:
-                res['password'] = 'x' * 10
-        else:
-            for val in res:
-                if 'password' in val:
-                    val['password'] = 'x' * 10
-        return res
-
-    def search_rec_name(self, name, clause):
-        ids = self.search([
+    @classmethod
+    def search_rec_name(cls, name, clause):
+        users = cls.search([
             ('login', '=', clause[2]),
             ], order=[])
-        if len(ids) == 1:
-            return [('id', '=', ids[0])]
-        return [(self._rec_name,) + clause[1:]]
+        if len(users) == 1:
+            return [('id', '=', users[0].id)]
+        return [(cls._rec_name,) + tuple(clause[1:])]
 
-    def copy(self, ids, default=None):
+    @classmethod
+    def copy(cls, users, default=None):
         if default is None:
             default = {}
         default = default.copy()
 
-        int_id = False
-        if isinstance(ids, (int, long)):
-            int_id = True
-            ids = [ids]
-
         default['password'] = ''
 
-        new_ids = []
-        for user in self.browse(ids):
+        new_users = []
+        for user in users:
             default['login'] = user.login + ' (copy)'
-            new_id = super(User, self).copy(user.id, default)
-            new_ids.append(new_id)
+            new_user, = super(User, cls).copy([user], default)
+            new_users.append(new_user)
+        return new_users
 
-        if int_id:
-            return new_ids[0]
-        return new_ids
-
-    def _get_preferences(self, user, context_only=False):
+    @classmethod
+    def _get_preferences(cls, user, context_only=False):
         pool = Pool()
-        model_data_obj = pool.get('ir.model.data')
-        action_obj = pool.get('ir.action')
-        config_item = pool.get('ir.module.module.config_wizard.item')
+        ModelData = pool.get('ir.model.data')
+        Action = pool.get('ir.action')
+        ConfigItem = pool.get('ir.module.module.config_wizard.item')
 
         res = {}
         if context_only:
-            fields = self._context_fields
+            fields = cls._context_fields
         else:
-            fields = self._preferences_fields + self._context_fields
+            fields = cls._preferences_fields + cls._context_fields
         for field in fields:
-            if self._columns[field]._type in ('many2one',):
+            if cls._fields[field]._type in ('many2one',):
                 if field == 'language':
                     if user.language:
                         res['language'] = user.language.code
                     else:
                         res['language'] = CONFIG['language']
                 else:
-                    res[field] = user[field].id
-                    if user[field].id:
-                        res[field + '.rec_name'] = user[field].rec_name
-            elif self._columns[field]._type in ('one2many', 'many2many'):
-                res[field] = [x.id for x in user[field]]
+                    res[field] = None
+                    if getattr(user, field):
+                        res[field] = getattr(user, field).id
+                        res[field + '.rec_name'] = \
+                            getattr(user, field).rec_name
+            elif cls._fields[field]._type in ('one2many', 'many2many'):
+                res[field] = [x.id for x in getattr(user, field)]
                 if field == 'actions' and user.login == 'admin':
-                    config_wizard_id = model_data_obj.get_id('ir',
+                    config_wizard_id = ModelData.get_id('ir',
                         'act_module_config_wizard')
-                    action_id = action_obj.get_action_id(config_wizard_id)
+                    action_id = Action.get_action_id(config_wizard_id)
                     if action_id in res[field]:
                         res[field].remove(action_id)
-                    if config_item.search([
+                    if ConfigItem.search([
                                 ('state', '=', 'open'),
                                 ]):
                         res[field].insert(0, action_id)
             else:
-                res[field] = user[field]
+                res[field] = getattr(user, field)
 
         if user.language:
             date = user.language.date
@@ -327,52 +333,57 @@ class User(ModelSQL, ModelView):
             }
         return res
 
-    @Cache('res_user.get_preferences')
-    def get_preferences(self, context_only=False):
+    @classmethod
+    def get_preferences(cls, context_only=False):
+        key = (Transaction().user, context_only)
+        preferences = cls._get_preferences_cache.get(key)
+        if preferences is not None:
+            return preferences
         user = Transaction().user
         with Transaction().set_user(0):
-            user = self.browse(user)
-        return self._get_preferences(user, context_only=context_only)
+            user = cls(user)
+        preferences = cls._get_preferences(user, context_only=context_only)
+        cls._get_preferences_cache.set(key, preferences)
+        return preferences
 
-    def set_preferences(self, values, old_password=False):
+    @classmethod
+    def set_preferences(cls, values, old_password=False):
         '''
         Set user preferences.
-
-        :param values: a dictionary with values
-        :param old_password: the previous password if password is in values
         '''
         pool = Pool()
-        lang_obj = pool.get('ir.lang')
+        Lang = pool.get('ir.lang')
         values_clean = values.copy()
-        fields = self._preferences_fields + self._context_fields
+        fields = cls._preferences_fields + cls._context_fields
         user_id = Transaction().user
+        with Transaction().set_user(0):
+            user = cls(user_id)
         for field in values:
             if field not in fields or field == 'groups':
                 del values_clean[field]
             if field == 'password':
-                with Transaction().set_user(0):
-                    user = self.browse(user_id)
-                    if not self.get_login(user.login, old_password):
-                        self.raise_user_error('wrong_password')
+                if not cls.get_login(user.login, old_password):
+                    cls.raise_user_error('wrong_password')
             if field == 'language':
-                lang_ids = lang_obj.search([
+                langs = Lang.search([
                     ('code', '=', values['language']),
                     ])
-                if lang_ids:
-                    values_clean['language'] = lang_ids[0]
+                if langs:
+                    values_clean['language'] = langs[0].id
                 else:
                     del values_clean['language']
         with Transaction().set_user(0):
-            self.write(user_id, values_clean)
+            cls.write([user], values_clean)
 
-    def get_preferences_fields_view(self):
+    @classmethod
+    def get_preferences_fields_view(cls):
         pool = Pool()
-        model_data_obj = pool.get('ir.model.data')
-        lang_obj = pool.get('ir.lang')
-        action_obj = pool.get('ir.action')
+        ModelData = pool.get('ir.model.data')
+        Lang = pool.get('ir.lang')
+        Action = pool.get('ir.action')
 
-        view_id = model_data_obj.get_id('res', 'user_view_form_preferences')
-        res = self.fields_view_get(view_id=view_id)
+        view_id = ModelData.get_id('res', 'user_view_form_preferences')
+        res = cls.fields_view_get(view_id=view_id)
         res = copy.deepcopy(res)
         for field in res['fields']:
             if field not in ('groups', 'language_direction'):
@@ -389,57 +400,61 @@ class User(ModelSQL, ModelView):
 
         if 'language' in res['fields']:
             selection = convert2selection(res['fields'], 'language')
-            lang_ids = lang_obj.search(['OR',
+            langs = Lang.search(['OR',
                     ('translatable', '=', True),
                     ('code', '=', CONFIG['language']),
                     ])
+            lang_ids = [l.id for l in langs]
             with Transaction().set_context(translate_name=True):
-                for lang in lang_obj.browse(lang_ids):
+                for lang in Lang.browse(lang_ids):
                     selection.append((lang.code, lang.name))
         if 'action' in res['fields']:
             selection = convert2selection(res['fields'], 'action')
             selection.append((None, ''))
-            action_ids = action_obj.search([])
-            for action in action_obj.browse(action_ids):
+            actions = Action.search([])
+            for action in actions:
                 selection.append((action.id, action.rec_name))
         if 'menu' in res['fields']:
             selection = convert2selection(res['fields'], 'menu')
-            action_ids = action_obj.search([
+            actions = Action.search([
                     ('usage', '=', 'menu'),
                     ])
-            for action in action_obj.browse(action_ids):
+            for action in actions:
                 selection.append((action.id, action.rec_name))
         return res
 
-    @Cache('res_user.get_groups')
-    def get_groups(self):
+    @classmethod
+    def get_groups(cls):
         '''
         Return a list of group ids for the user
-
-        :return: a list of group ids
         '''
-        return self.read(Transaction().user, ['groups'])['groups']
+        user = Transaction().user
+        groups = cls._get_groups_cache.get(user)
+        if groups is not None:
+            return groups
+        groups = cls.read([Transaction().user], ['groups'])[0]['groups']
+        cls._get_groups_cache.set(user, groups)
+        return groups
 
-    @Cache('res_user._get_login')
-    def _get_login(self, login):
+    @classmethod
+    def _get_login(cls, login):
+        result = cls._get_groups_cache.get(login)
+        if result:
+            return result
         cursor = Transaction().cursor
         cursor.execute('SELECT id, password, salt ' \
-                'FROM "' + self._table + '" '
+                'FROM "' + cls._table + '" '
                 'WHERE login = %s AND active', (login,))
-        res = cursor.fetchone()
-        if not res:
-            return None, None, None
-        return res
+        result = cursor.fetchone() or (None, None, None)
+        cls._get_groups_cache.set(login, result)
+        return result
 
-    def get_login(self, login, password):
+    @classmethod
+    def get_login(cls, login, password):
         '''
         Return user id if password matches
-
-        :param login: the login name
-        :param password: the password
-        :return: integer
         '''
-        user_id, user_password, salt = self._get_login(login)
+        user_id, user_password, salt = cls._get_login(login)
         if not user_id:
             return 0
         password += salt or ''
@@ -451,118 +466,101 @@ class User(ModelSQL, ModelView):
             password_sha = sha.new(password).hexdigest()
         cursor = Transaction().cursor
         if password_sha == user_password:
-            cursor.execute('UPDATE "' + self._table + '" '
+            cursor.execute('UPDATE "' + cls._table + '" '
                 'SET login_try = 0 '
                 'WHERE login = %s', (login,))
             return user_id
-        cursor.execute('UPDATE "' + self._table + '" '
+        cursor.execute('UPDATE "' + cls._table + '" '
             'SET login_try = login_try + 1 '
             'WHERE login = %s', (login,))
-        cursor.execute('SELECT login_try FROM "' + self._table + '" '
+        cursor.execute('SELECT login_try FROM "' + cls._table + '" '
             'WHERE login = %s', (login,))
         login_try, = cursor.fetchone()
         time.sleep(2 ** login_try)
         return 0
 
-User()
-
 
 class UserAction(ModelSQL):
     'User - Action'
-    _name = 'res.user-ir.action'
+    __name__ = 'res.user-ir.action'
     user = fields.Many2One('res.user', 'User', ondelete='CASCADE', select=True,
         required=True)
     action = fields.Many2One('ir.action', 'Action', ondelete='CASCADE',
         select=True, required=True)
 
-    def _convert_values(self, values):
+    @staticmethod
+    def _convert_values(values):
         pool = Pool()
-        action_obj = pool.get('ir.action')
+        Action = pool.get('ir.action')
         values = values.copy()
         if values.get('action'):
-            values['action'] = action_obj.get_action_id(values['action'])
+            values['action'] = Action.get_action_id(values['action'])
         return values
 
-    def create(self, values):
-        values = self._convert_values(values)
-        return super(UserAction, self).create(values)
+    @classmethod
+    def create(cls, values):
+        values = cls._convert_values(values)
+        return super(UserAction, cls).create(values)
 
-    def write(self, ids, values):
-        values = self._convert_values(values)
-        return super(UserAction, self).write(values)
-
-UserAction()
+    @classmethod
+    def write(cls, records, values):
+        values = cls._convert_values(values)
+        super(UserAction, cls).write(records, values)
 
 
 class UserGroup(ModelSQL):
     'User - Group'
-    _name = 'res.user-res.group'
-    _description = __doc__
+    __name__ = 'res.user-res.group'
     user = fields.Many2One('res.user', 'User', ondelete='CASCADE', select=True,
             required=True)
     group = fields.Many2One('res.group', 'Group', ondelete='CASCADE',
             select=True, required=True)
 
-    def init(self, module_name):
+    @classmethod
+    def __register__(cls, module_name):
         cursor = Transaction().cursor
         # Migration from 1.0 table name change
-        TableHandler.table_rename(cursor, 'res_group_user_rel', self._table)
+        TableHandler.table_rename(cursor, 'res_group_user_rel', cls._table)
         TableHandler.sequence_rename(cursor, 'res_group_user_rel_id_seq',
-                self._table + '_id_seq')
+                cls._table + '_id_seq')
         # Migration from 2.0 uid and gid rename into user and group
-        table = TableHandler(cursor, self, module_name)
+        table = TableHandler(cursor, cls, module_name)
         table.column_rename('uid', 'user')
         table.column_rename('gid', 'group')
-        super(UserGroup, self).init(module_name)
-
-UserGroup()
+        super(UserGroup, cls).__register__(module_name)
 
 
-class Group(ModelSQL, ModelView):
-    "Group"
-    _name = "res.group"
-    users = fields.Many2Many('res.user-res.group', 'group', 'user', 'Users')
-
-Group()
-
-
-class Warning(ModelSQL, ModelView):
+class Warning_(ModelSQL, ModelView):
     'User Warning'
-    _name = 'res.user.warning'
-    _description = __doc__
+    __name__ = 'res.user.warning'
 
     user = fields.Many2One('res.user', 'User', required=True, select=True)
     name = fields.Char('Name', required=True, select=True)
     always = fields.Boolean('Always')
 
-    def check(self, warning_name):
+    @classmethod
+    def check(cls, warning_name):
         user = Transaction().user
         if not user:
             return False
-        warning_ids = self.search([
+        warnings = cls.search([
             ('user', '=', user),
             ('name', '=', warning_name),
             ])
-        if not warning_ids:
+        if not warnings:
             return True
-        warnings = self.browse(warning_ids)
-        self.delete([x.id for x in warnings if not x.always])
+        cls.delete([x.id for x in warnings if not x.always])
         return False
 
-Warning()
-
 
 class UserConfigStart(ModelView):
     'User Config Init'
-    _name = 'res.user.config.start'
-    _description = __doc__
-
-UserConfigStart()
+    __name__ = 'res.user.config.start'
 
 
 class UserConfig(Wizard):
     'Configure users'
-    _name = 'res.user.config'
+    __name__ = 'res.user.config'
 
     start = StateView('res.user.config.start',
         'res.user_config_start_view_form', [
@@ -576,20 +574,6 @@ class UserConfig(Wizard):
             ])
     add = StateTransition()
 
-    def transition_add(self, session):
-        pool = Pool()
-        user_obj = pool.get('res.user')
-        values = session.data['user'].copy()
-        for fname in values.keys():
-            if fname == 'id':
-                del values[fname]
-                continue
-            field = user_obj._columns[fname]
-            if field._type == 'one2many':
-                values[fname] = [('create', v) for v in values[fname]]
-            elif field._type == 'many2many':
-                values[fname] = [('set', [v['id'] for v in values[fname]])]
-        user_obj.create(values)
+    def transition_add(self):
+        self.user.save()
         return 'user'
-
-UserConfig()
