@@ -93,10 +93,6 @@ class TrytonServer(object):
             with open(CONFIG['pidfile'], 'w') as fd_pid:
                 fd_pid.write("%d" % (os.getpid()))
 
-        if CONFIG['psyco']:
-            import psyco
-            psyco.full()
-
         if not CONFIG["db_name"] \
                 and bool(CONFIG['init'] or CONFIG['update']):
             raise Exception('Missing database option!')
@@ -142,17 +138,30 @@ class TrytonServer(object):
 
         for db_name in CONFIG['db_name']:
             if init[db_name]:
-                while True:
-                    password = getpass('Admin Password for %s: ' % db_name)
-                    password2 = getpass('Admin Password Confirmation: ')
-                    if password != password2:
-                        sys.stderr.write('Admin Password Confirmation ' \
-                                'doesn\'t match Admin Password!\n')
-                        continue
-                    if not password:
-                        sys.stderr.write('Admin Password is required!\n')
-                        continue
-                    break
+                # try to read password from environment variable
+                # TRYTONPASSFILE, empty TRYTONPASSFILE ignored
+                passpath = os.getenv('TRYTONPASSFILE')
+                password = ''
+                if passpath:
+                    try:
+                        with open(passpath) as passfile:
+                            password = passfile.readline()[:-1]
+                    except Exception, err:
+                        sys.stderr.write('Can not read password ' \
+                                'from "%s": "%s"\n' % (passpath, err))
+
+                if not password:
+                    while True:
+                        password = getpass('Admin Password for %s: ' % db_name)
+                        password2 = getpass('Admin Password Confirmation: ')
+                        if password != password2:
+                            sys.stderr.write('Admin Password Confirmation ' \
+                                    'doesn\'t match Admin Password!\n')
+                            continue
+                        if not password:
+                            sys.stderr.write('Admin Password is required!\n')
+                            continue
+                        break
 
                 database = Database(db_name).connect()
                 cursor = database.cursor()
@@ -189,11 +198,11 @@ class TrytonServer(object):
                     try:
                         if 'ir.cron' not in pool.object_name_list():
                             continue
-                        cron_obj = pool.get('ir.cron')
+                        Cron = pool.get('ir.cron')
                     finally:
                         pool.lock.release()
                     thread = threading.Thread(
-                            target=cron_obj.run,
+                            target=Cron.run,
                             args=(dbname,), kwargs={})
                     thread.start()
                     threads[dbname] = thread
