@@ -5,15 +5,9 @@
 
 import unittest
 import datetime
-import socket
-import urllib
 
-from trytond.config import CONFIG
 from trytond.tools import reduce_ids, safe_eval, datetime_strftime, \
-        reduce_domain
-from trytond.tests.test_tryton import (POOL, DB_NAME, USER, CONTEXT,
-    install_module)
-from trytond.transaction import Transaction
+    reduce_domain
 
 
 class ToolsTestCase(unittest.TestCase):
@@ -53,19 +47,29 @@ class ToolsTestCase(unittest.TestCase):
         '''
         Test reduce_ids complex list.
         '''
-        self.assert_(('(((id >= %s) AND (id <= %s)) OR ' \
-                '(id IN (%s,%s,%s,%s,%s)))', [0, 14, 25, 26, 27, 28, 29]) == \
-                reduce_ids('id', range(10) + map(lambda x: x + 25, range(5)) + \
-                map(lambda x: x + 5, range(10))))
+        self.assertEqual(('(((id >= %s) AND (id <= %s)) OR '
+                '(id IN (%s,%s,%s,%s,%s)))', [0, 14, 25, 26, 27, 28, 29]),
+            reduce_ids('id', range(10) + map(lambda x: x + 25, range(5))
+                + map(lambda x: x + 5, range(10))))
 
     def test0050reduce_ids_complex_small_continue(self):
         '''
         Test reduce_ids complex list with small continue.
         '''
+        self.assertEqual(('(((id >= %s) AND (id <= %s)) '
+                'OR (id IN (%s,%s,%s,%s)))', [1, 12, 15, 18, 19, 21]),
+            reduce_ids('id', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15, 18,
+                    19, 21]))
+
+    def test0055reduce_ids_float(self):
+        '''
+        Test reduce_ids with integer as float.
+        '''
         self.assert_(('(((id >= %s) AND (id <= %s)) OR (id IN (%s,%s,%s,%s)))',
-            [1, 12, 15, 18, 19, 21]) == \
-                    reduce_ids('id', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 15,
-                        18, 19, 21]))
+                [1.0, 12.0, 15.0, 18.0, 19.0, 21.0]) == \
+                    reduce_ids('id', [1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0,
+                    9.0, 10.0, 11.0, 12.0, 15.0, 18.0, 19.0, 21.0]))
+        self.assertRaises(AssertionError, reduce_ids, 'id', [1.1])
 
     def test0060safe_eval_builtin(self):
         '''
