@@ -1,22 +1,22 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
 
-__all__ = ['Wizard', 'StateView', 'StateTransition', 'StateAction', 'Button',
-    'Session']
+__all__ = ['Wizard', 'StateView', 'StateTransition', 'StateAction', 'Button']
 
 try:
     import simplejson as json
 except ImportError:
     import json
+import copy
 
-from trytond.pool import Pool
+from trytond.pool import Pool, PoolMeta
 from trytond.transaction import Transaction
 from trytond.error import WarningErrorMixin
 from trytond.url import URLMixin
 from trytond.protocols.jsonrpc import object_hook, JSONEncoder
 from trytond.model.fields import states_validate
 from trytond.pyson import PYSONEncoder
-from trytond.model.browse import BrowseRecordNull
+from trytond.rpc import RPC
 
 
 class Button(object):
@@ -46,6 +46,7 @@ class State(object):
     '''
     A State of a wizard.
     '''
+    name = None
 
 
 class StateView(State):
@@ -69,34 +70,34 @@ class StateView(State):
         '''
         Returns the view definition
         '''
-        model_obj = Pool().get(self.model_name)
-        model_data_obj = Pool().get('ir.model.data')
+        Model_ = Pool().get(self.model_name)
+        ModelData = Pool().get('ir.model.data')
         module, fs_id = self.view.split('.')
-        view_id = model_data_obj.get_id(module, fs_id)
-        return model_obj.fields_view_get(view_id=view_id, view_type='form')
+        view_id = ModelData.get_id(module, fs_id)
+        return Model_.fields_view_get(view_id=view_id, view_type='form')
 
-    def get_defaults(self, wizard, session, state_name, fields):
+    def get_defaults(self, wizard, state_name, fields):
         '''
         Returns defaults values for the fields
         '''
-        model_obj = Pool().get(self.model_name)
-        defaults = model_obj.default_get(fields)
+        Model_ = Pool().get(self.model_name)
+        defaults = Model_.default_get(fields)
         default = getattr(wizard, 'default_%s' % state_name, None)
         if default:
-            defaults.update(default(session, fields))
+            defaults.update(default(fields))
         return defaults
 
     def get_buttons(self, wizard, state_name):
         '''
         Returns button definitions translated
         '''
-        translation_obj = Pool().get('ir.translation')
+        Translation = Pool().get('ir.translation')
 
         def translation_key(button):
-            return (','.join((wizard._name, state_name, button.state)),
+            return (','.join((wizard.__name__, state_name, button.state)),
                 'wizard_button', Transaction().language, button.string)
         translation_keys = [translation_key(button) for button in self.buttons]
-        translations = translation_obj.get_sources(translation_keys)
+        translations = Translation.get_sources(translation_keys)
         encoder = PYSONEncoder()
         result = []
         for button in self.buttons:
@@ -132,131 +133,48 @@ class StateAction(StateTransition):
     def get_action(self):
         "Returns action definition"
         pool = Pool()
-        model_data_obj = pool.get('ir.model.data')
-        action_obj = pool.get('ir.action')
+        ModelData = pool.get('ir.model.data')
+        Action = pool.get('ir.action')
         module, fs_id = self.action_id.split('.')
-        action_id = action_obj.get_action_id(
-            model_data_obj.get_id(module, fs_id))
-        action = action_obj.browse(action_id)
-        return action_obj.get_action_values(action.type, action.id)
-
-
-class _SessionRecord(object):
-    '''
-    A record of a wizard form.
-    '''
-    # Declared in class to prevent:
-    # 'maximum recursion depth exceeded in __subclasscheck__'
-    _model = None
-    _data = None
-    __cache = None
-
-    def __init__(self, model, data):
-        self._model = model
-        self._data = data
-        self.__cache = {}
-        self.dirty = False
-
-    def __getattr__(self, name):
-        if name in self.__cache:
-            return self.__cache[name]
-        field = self._model._columns[name]
-        data = self._data.get(name, None)
-        target_obj = None
-        if hasattr(field, 'model_name'):
-            target_obj = Pool().get(field.model_name)
-        elif hasattr(field, 'get_target'):
-            target_obj = field.get_target()
-        if data:
-            if target_obj:
-                def instance(data):
-                    if isinstance(data, dict):
-                        return _SessionRecord(target_obj, data)
-                    return target_obj.browse(data)
-                if isinstance(data, list):
-                    data = [instance(x) for x in data]
-                else:
-                    data = instance(data)
-        elif target_obj:
-            return BrowseRecordNull()
-        self.__cache[name] = data
-        return data
-
-    def __setattr__(self, name, value):
-        if (self._model is not None
-                and (name in self._model._columns
-                    or name in self._model._inherit_fields)):
-            self.__cache.pop(name, None)
-            self._data[name] = value
-            self.dirty = True
-        else:
-            super(_SessionRecord, self).__setattr__(name, value)
-
-
-class Session(object):
-    '''
-    A wizard session.
-    '''
-
-    def __init__(self, wizard, session_id):
-        pool = Pool()
-        session_obj = pool.get('ir.session.wizard')
-        self._session = session_obj.browse(session_id)
-        self.data = json.loads(self._session.data.encode('utf-8'),
-            object_hook=object_hook)
-        for state_name, state in wizard.states.iteritems():
-            if isinstance(state, StateView):
-                model = pool.get(state.model_name)
-                self.data.setdefault(state_name, {})
-                setattr(self, state_name,
-                    _SessionRecord(model, self.data[state_name]))
-
-    @property
-    def dirty(self):
-        return any(getattr(self, state_name).dirty
-            for state_name in self.data)
-
-    def save(self):
-        "Save the session in database"
-        session_obj = Pool().get('ir.session.wizard')
-        if self.dirty:
-            session_obj.write(self._session.id, {
-                    'data': json.dumps(self.data, cls=JSONEncoder),
-                    })
+        action_id = Action.get_action_id(
+            ModelData.get_id(module, fs_id))
+        action = Action(action_id)
+        return Action.get_action_values(action.type, [action.id])[0]
 
 
 class Wizard(WarningErrorMixin, URLMixin):
-    _name = ""
+    __metaclass__ = PoolMeta
     start_state = 'start'
     end_state = 'end'
 
-    def __new__(cls):
-        Pool.register(cls, type='wizard')
+    @classmethod
+    def __setup__(cls):
+        cls.__rpc__ = {
+            'create': RPC(readonly=False),
+            'delete': RPC(readonly=False),
+            'execute': RPC(readonly=False),
+            }
+        cls._error_messages = {}
 
-    def __init__(self):
-        super(Wizard, self).__init__()
-        self._rpc = {
-            'create': True,
-            'delete': True,
-            'execute': True,
-        }
-        self._error_messages = {}
+        # Copy states
+        for attr in dir(cls):
+            if isinstance(getattr(cls, attr), State):
+                setattr(cls, attr, copy.deepcopy(getattr(cls, attr)))
 
-    @property
-    def states(self):
-        result = {}
-        for attr in dir(self):
-            if attr == 'states':
-                continue
-            if isinstance(getattr(self, attr), State):
-                result[attr] = getattr(self, attr)
-        return result
+    @classmethod
+    def __post_setup__(cls):
+        # Set states
+        cls.states = {}
+        for attr in dir(cls):
+            if isinstance(getattr(cls, attr), State):
+                cls.states[attr] = getattr(cls, attr)
 
-    def init(self, module_name):
+    @classmethod
+    def __register__(cls, module_name):
         pool = Pool()
-        translation_obj = pool.get('ir.translation')
+        Translation = pool.get('ir.translation')
         cursor = Transaction().cursor
-        for state_name, state in self.states.iteritems():
+        for state_name, state in cls.states.iteritems():
             if isinstance(state, StateView):
                 for button in state.buttons:
                     cursor.execute('SELECT id, name, src '
@@ -266,16 +184,16 @@ class Wizard(WarningErrorMixin, URLMixin):
                             'AND type = %s '
                             'AND name = %s',
                         (module_name, 'en_US', 'wizard_button',
-                            self._name + ',' + state_name + ',' +
+                            cls.__name__ + ',' + state_name + ',' +
                             button.state))
                     res = cursor.dictfetchall()
-                    src_md5 = translation_obj.get_src_md5(button.string)
+                    src_md5 = Translation.get_src_md5(button.string)
                     if not res:
                         cursor.execute('INSERT INTO ir_translation '
                             '(name, lang, type, src, src_md5, value, module, '
                                 'fuzzy) '
                             'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                            (self._name + ',' + state_name + ',' +
+                            (cls.__name__ + ',' + state_name + ',' +
                                 button.state,
                                 'en_US', 'wizard_button', button.string,
                                 src_md5, '', module_name, False))
@@ -289,31 +207,34 @@ class Wizard(WarningErrorMixin, URLMixin):
             'WHERE lang = %s '
                 'AND type = %s '
                 'AND name = %s',
-            ('en_US', 'error', self._name))
+            ('en_US', 'error', cls.__name__))
         trans_error = {}
         for trans in cursor.dictfetchall():
             trans_error[trans['src']] = trans
 
-        for error in self._error_messages.values():
+        for error in cls._error_messages.values():
             if error not in trans_error:
-                error_md5 = translation_obj.get_src_md5(error)
+                error_md5 = Translation.get_src_md5(error)
                 cursor.execute('INSERT INTO ir_translation '
                     '(name, lang, type, src, src_md5,  value, module, fuzzy) '
                     'VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
-                    (self._name, 'en_US', 'error', error, error_md5, '',
+                    (cls.__name__, 'en_US', 'error', error, error_md5, '',
                         module_name, False))
 
-    def create(self):
+    @classmethod
+    def create(cls):
         "Create a session"
-        session_obj = Pool().get('ir.session.wizard')
-        return (session_obj.create({}), self.start_state, self.end_state)
+        Session = Pool().get('ir.session.wizard')
+        return (Session.create({}).id, cls.start_state, cls.end_state)
 
-    def delete(self, session_id):
+    @classmethod
+    def delete(cls, session_id):
         "Delete the session"
-        session_obj = Pool().get('ir.session.wizard')
-        session_obj.delete(session_id)
+        Session = Pool().get('ir.session.wizard')
+        Session.delete([Session(session_id)])
 
-    def execute(self, session, data, state_name):
+    @classmethod
+    def execute(cls, session, data, state_name):
         '''
         Execute the wizard state.
 
@@ -328,22 +249,25 @@ class Wizard(WarningErrorMixin, URLMixin):
                 - ``defaults``: a dictionary with default values
                 - ``buttons``: a list of buttons
         '''
+        wizard = cls(session)
+        for key, values in data.iteritems():
+            record = getattr(wizard, key)
+            for field, value in values.iteritems():
+                if field == 'id':
+                    continue
+                setattr(record, field, value)
+        return wizard._execute(state_name)
+
+    def _execute(self, state_name):
         if state_name == self.end_state:
             return {}
-        if isinstance(session, (int, long)):
-            session = Session(self, session)
-        for key, value in data.iteritems():
-            prev_data = session.data[key].copy()
-            session.data[key].update(value)
-            if prev_data != session.data[key]:
-                getattr(session, key).dirty = True
 
         state = self.states[state_name]
         result = {}
 
         if isinstance(state, StateView):
             view = state.get_view()
-            defaults = state.get_defaults(self, session, state_name,
+            defaults = state.get_defaults(self, state_name,
                 view['fields'].keys())
             buttons = state.get_buttons(self, state_name)
             result['view'] = {
@@ -358,13 +282,40 @@ class Wizard(WarningErrorMixin, URLMixin):
                 action = state.get_action()
                 do = getattr(self, 'do_%s' % state_name, None)
                 if do:
-                    do_result = do(session, action)
+                    do_result = do(action)
                 else:
                     do_result = action, {}
             transition = getattr(self, 'transition_%s' % state_name, None)
             if transition:
-                result = self.execute(session, {}, transition(session))
+                result = self._execute(transition())
             if do_result:
                 result.setdefault('actions', []).append(do_result)
-        session.save()
+        self._save()
         return result
+
+    def __init__(self, session_id):
+        pool = Pool()
+        Session = pool.get('ir.session.wizard')
+        self._session_id = session_id
+        session = Session(session_id)
+        data = json.loads(session.data.encode('utf-8'),
+            object_hook=object_hook)
+        for state_name, state in self.states.iteritems():
+            if isinstance(state, StateView):
+                Target = pool.get(state.model_name)
+                data.setdefault(state_name, {})
+                setattr(self, state_name, Target(**data[state_name]))
+
+    def _save(self):
+        "Save the session in database"
+        Session = Pool().get('ir.session.wizard')
+        data = {}
+        for state_name, state in self.states.iteritems():
+            if isinstance(state, StateView):
+                data[state_name] = getattr(self, state_name)._default_values
+        session = Session(self._session_id)
+        data = json.dumps(data, cls=JSONEncoder)
+        if data != session.data.encode('utf-8'):
+            Session.write([session], {
+                    'data': data,
+                    })
